@@ -31,18 +31,24 @@
 #include <sys/time.h>
 #include <regex.h>
 
+#define RUNPARTS_NORMAL 0
+#define RUNPARTS_ERE 1
+#define RUNPARTS_LSBSYSINIT 100
+
 int test_mode = 0;
 int list_mode = 0;
 int verbose_mode = 0;
 int report_mode = 0;
 int reverse_mode = 0;
 int exitstatus = 0;
-int lsbsysinit_mode = 0;
+int regex_mode = 0;
 int exit_on_error_mode = 0;
 
 int argcount = 0, argsize = 0;
 char **args = 0;
 
+char *custom_ere;
+
 void error(char *format, ...)
 {
   va_list ap;
@@ -62,7 +68,7 @@ void version()
   fprintf(stderr, "Debian run-parts program, version " PACKAGE_VERSION
 	  "\nCopyright (C) 1994 Ian Jackson, Copyright (C) 1996 Jeff Noxon.\n"
 	  "Copyright (C) 1996,1997,1998,1999 Guy Maor\n"
-	  "Copyright (C) 2002, 2003, 2004, 2005 Clint Adams\n"
+	  "Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Clint Adams\n"
 	  "This is free software; see the GNU General Public License version 2\n"
 	  "or later for copying conditions.  There is NO warranty.\n");
   exit(0);
@@ -81,6 +87,7 @@ void usage()
 	  "      --exit-on-error exit as soon as a script returns with a non-zero exit\n"
 	  "                      code.\n"
 	  "      --lsbsysinit    validate filenames based on LSB sysinit specs.\n"
+	  "      --regex=PATTERN validate filenames based on POSIX ERE pattern PATTERN.\n"
 	  "  -u, --umask=UMASK   sets umask to UMASK (octal), default is 022.\n"
 	  "  -a, --arg=ARGUMENT  pass ARGUMENT to scripts, use once for each argument.\n"
 	  "  -V, --version       output version information and exit.\n"
@@ -125,11 +132,17 @@ void add_argument(char *newarg)
 int valid_name(const struct dirent *d)
 {
   char *c = d->d_name;
-  regex_t hierre, tradre, excsre, classicalre;
+  regex_t hierre, tradre, excsre, classicalre, customre;
 
   /* The regcomps should be moved to program init */
-
-  if (lsbsysinit_mode) {
+  if (regex_mode == RUNPARTS_ERE) {
+    if (regcomp(&customre, custom_ere, REG_EXTENDED | REG_NOSUB)) {
+      error("custom regex failure");
+      exit(1);
+    }
+    return !regexec(&customre, c, 0, NULL, 0);
+  }
+  else if (regex_mode == RUNPARTS_LSBSYSINIT) {
 
     if (regcomp
 	(&hierre, "^_?([a-z0-9_.]+-)+[a-z0-9]+$", REG_EXTENDED | REG_NOSUB)
@@ -394,7 +407,8 @@ int main(int argc, char *argv[])
       {"arg", 1, 0, 'a'},
       {"help", 0, 0, 'h'},
       {"version", 0, 0, 'V'},
-      {"lsbsysinit", 0, &lsbsysinit_mode, 1},
+      {"lsbsysinit", 0, &regex_mode, RUNPARTS_LSBSYSINIT},
+      {"regex", 1, &regex_mode, RUNPARTS_ERE},
       {"exit-on-error", 0, &exit_on_error_mode, 1},
       {0, 0, 0, 0}
     };
@@ -404,6 +418,9 @@ int main(int argc, char *argv[])
       break;
     switch (c) {
     case 0:
+      if(option_index==10) { /* hardcoding this will lead to trouble */
+        custom_ere = strdup(optarg);
+      }
       break;
     case 'u':
       set_umask();
